<?php

namespace HCM\Geral\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use HCM\Geral\Services\ComunicadoService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use HCM\Util\Messages;

class ComunicadoController extends Crud {

    protected $dataSourceName = 'getComunicado';
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, ComunicadoService $comunicadoService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->ComunicadoService = $comunicadoService;
        parent::__construct($dataSourceManager);
    }

    public function getComunicado(DTO\Request\Filter $request, DTO\Response $response) {
        
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $rows = $dataSet->getRows();
	    foreach($rows as $key => &$row){
            if(isset($row['NRVINCULOM'])){
                $row['NRVINCULOM'] = explode(",", $row['NRVINCULOM']);
                
                foreach($row['NRVINCULOM'] as $chave => $value)
                    $row['NRVINCULOM'][$chave] = intval($value);
                    
                $row['NMVINCULOM'] = explode(",", $row['NMVINCULOM']);
            }
            
            if(isset($row['NRESTRUTSIND'])){
                $row['NRESTRUTSIND'] = explode(",", $row['NRESTRUTSIND']);
                
                foreach($row['NRESTRUTSIND'] as $chave => $value)
                    $row['NRESTRUTSIND'][$chave] = intval($value);
                    
                $row['NMESTRUTSIND'] = explode(",", $row['NMESTRUTSIND']);
            }
            if(isset($row['NRESTRUTLEGAL'])){
                $row['NRESTRUTLEGAL'] = explode(",", $row['NRESTRUTLEGAL']);
                
                foreach($row['NRESTRUTLEGAL'] as $chave => $value)
                    $row['NRESTRUTLEGAL'][$chave] = intval($value);
                    
                $row['NMESTRUTLEGAL'] = explode(",", $row['NMESTRUTLEGAL']);
            }
            if(isset($row['NRESTRUTGEREN'])){
                $row['NRESTRUTGEREN'] = explode(",", $row['NRESTRUTGEREN']);
                
                foreach($row['NRESTRUTGEREN'] as $chave => $value)
                    $row['NRESTRUTGEREN'][$chave] = intval($value);
                    
                $row['NMESTRUTGEREN'] = explode(",", $row['NMESTRUTGEREN']);
            }
        }
        $response->addDataSet(new DataSet("getComunicado", $rows));         
    }

    
    public function setComunicado(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = 'gpe_comunicado';
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRCOMUNICADO', $row)) {
                    if ($row['NRCOMUNICADO'] === '') {
                        $row['NRCOMUNICADO'] = \Geral\Util\NovoCodigo::geraCodigo($row['NRORG'], 'GPE_COMUNICADO', 1, 12, true); 
                    }
                } else {
                    $row['NRCOMUNICADO'] = \Geral\Util\NovoCodigo::geraCodigo($row['NRORG'], 'GPE_COMUNICADO', 1, 12, true); 
                }
                    if(array_key_exists('NRVINCULOM', $row) && $row['NRVINCULOM'] <> ""){
                    $row['NRVINCULOM'] = implode(',',$row['NRVINCULOM']);
                    }
                    if(array_key_exists('NRESTRUTSIND', $row) && $row['NRESTRUTSIND']<> ""){
                    $row['NRESTRUTSIND'] = implode(',',$row['NRESTRUTSIND']);
                    }
                    if(array_key_exists('NRESTRUTLEGAL', $row) && $row['NRESTRUTLEGAL'] <> ""){
                    $row['NRESTRUTLEGAL'] = implode(',',$row['NRESTRUTLEGAL']);
                    }
                    if(array_key_exists('NRESTRUTGEREN', $row) && $row['NRESTRUTGEREN'] <> ""){
                    $row['NRESTRUTGEREN'] = implode(',',$row['NRESTRUTGEREN']);
                    }  
            $row['NRESTRUTLEGAL'] = isset($row['NRESTRUTLEGAL'])?$row['NRESTRUTLEGAL']:null;
            $row['NRESTRUTGEREN'] = isset($row['NRESTRUTGEREN'])?$row['NRESTRUTGEREN']:null;
            $row['NRESTRUTSIND'] = isset($row['NRESTRUTSIND'])?$row['NRESTRUTSIND']:null;
            $row['NRVINCULOM'] = isset($row['NRVINCULOM'])?$row['NRVINCULOM']:null;

            }
            
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    
    public function deleteComunicado(DTO\Request\DataSet $request, DTO\Response $response){
        parent::delete($request, $response);
    }

    public function enviarComunicado (DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            foreach ($request->getDataSet()->getRows() as $row) {
                $this->ComunicadoService->enviarComunicado($row);
            }
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('enviarComunicado', array(true)));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 