<?php

namespace HCM\SST\Controllers;

use HCM\SST\Services\ExameService;
use HCM\Util\Messages;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class ExameController extends Crud {

    protected $dataSourceName = 'sst_exame';
    protected $dataSourceName_get = 'sst_exame_get';
    protected $dataSourceName_item = 'sst_itemexame';
    protected $dataSourceName_refer = 'sst_referenciaitemexame';
    protected $dataSourceName_faixa = 'sst_faixarefitemexame';
    protected $dataSourceName_classif = 'sst_classifrefitemexame';

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, ExameService $exameService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->exameService = $exameService;
        parent::__construct($dataSourceManager);
    }

    public function getExame(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_get);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setExame(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NREXAME', $row)) {
                    if ($row['NREXAME'] === '') {
                        $row['NREXAME'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_EXAME', 1, 12, true);
                    }
                } else {
                    $row['NREXAME'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_EXAME', 1, 12, true);
                }
            }

            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteExame(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $this->exameService->vdExameHasItem($row['NREXAME']);
            }
            parent::delete($request, $response);
        } catch(\Exception $e) {
            if ($e->getCode() === Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    public function getItemExame(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_item);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setItemExame(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRITEMEXAME', $row)) {
                    if ($row['NRITEMEXAME'] === '') {
                        $row['NRITEMEXAME'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_ITEMEXAME', 1, 12, true);
                    }
                } else {
                    $row['NRITEMEXAME'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_ITEMEXAME', 1, 12, true);
                }
            }

            $this->dataSourceName = $this->dataSourceName_item;
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteItemExame(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $this->exameService->vdItemExameHasReferencia($row['NRITEMEXAME']);
            }
            $this->dataSourceName = $this->dataSourceName_item;
            parent::delete($request, $response);
        } catch(\Exception $e) {
            if ($e->getCode() === Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    public function getReferenciaItemExame(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_refer);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setReferenciaItemExame(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRREFERENCIAITEMEXAME', $row)) {
                    if ($row['NRREFERENCIAITEMEXAME'] === '') {
                        $row['NRREFERENCIAITEMEXAME'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_REFERENCIAITEMEXAME', 1, 12, true);
                    }
                } else {
                    $row['NRREFERENCIAITEMEXAME'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_REFERENCIAITEMEXAME', 1, 12, true);
                }
            }

            $this->dataSourceName = $this->dataSourceName_refer;
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteReferenciaItemExame(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $this->exameService->vdReferenciaHasFaixas($row['NRREFERENCIAITEMEXAME']);
                $this->exameService->vdReferenciaHasClassificacoes($row['NRREFERENCIAITEMEXAME']);
            }
            $this->dataSourceName = $this->dataSourceName_refer;
            parent::delete($request, $response);
        } catch(\Exception $e) {
            if ($e->getCode() === Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    public function getFaixaRefItemExame(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_faixa);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setFaixaRefItemExame(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRFAIXAREFITEMEXAME', $row)) {
                    if ($row['NRFAIXAREFITEMEXAME'] === '') {
                        $row['NRFAIXAREFITEMEXAME'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_FAIXAREFITEMEXAME', 1, 12, true);
                    }
                } else {
                    $row['NRFAIXAREFITEMEXAME'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_FAIXAREFITEMEXAME', 1, 12, true);
                }
            }

            $this->dataSourceName = $this->dataSourceName_faixa;
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteFaixaRefItemExame(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = $this->dataSourceName_faixa;
        parent::delete($request, $response);
    }

    public function getClassifRefItemExame(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_classif);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setClassifRefItemExame(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRCLASSIFREFITEMEXAME', $row)) {
                    if ($row['NRCLASSIFREFITEMEXAME'] === '') {
                        $row['NRCLASSIFREFITEMEXAME'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_CLASSIFREFITEMEXAME', 1, 12, true);
                    }
                } else {
                    $row['NRCLASSIFREFITEMEXAME'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_CLASSIFREFITEMEXAME', 1, 12, true);
                }
            }

            $this->dataSourceName = $this->dataSourceName_classif;
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteClassifRefItemExame(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = $this->dataSourceName_classif;
        parent::delete($request, $response);
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 