<?php

namespace HCM\Util\NewFileServer;

use Doctrine\ORM\EntityManager;

use HCM\Util\NewFileServer\FileServerClasses\FSMigration\FileServer\Files\FSFilesClient;
use HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use HCM\Util\Messages;

class NewFileServer {
    /** @var FSFilesClient */
    protected $client;

    /** @var OrganizacaoRepository */
    protected $organizacaoRepository;
    
    /** @var EntityManager */
    protected $entityManager;

    public function __construct(
        HCMParameters   $hcmParameters,
        EntityManager   $entityManager,
        string          $baseUrl
    ) {
        $this->hcmParameters = $hcmParameters;
        $this->entityManager = $entityManager;
        
        $key = $this->getAPIKey();
        $nrOrg = $this->hcmParameters->getNrOrgTrabalho();
        
        $this->client = new FSFilesClient($nrOrg, $key, $baseUrl);
    }

    public function getAPIKey(): string {
        $nrOrg = $this->hcmParameters->getNrOrgTrabalho();
        
        $organizacao = $this->entityManager->getRepository(Repositories::ORGANIZACAO)->findOneBy(array('nrorg' => $nrOrg));
        if (empty($organizacao)) {
            throw new \Exception(Messages::ORG_NOT_FOUND);
        }
        
        $key = $organizacao->getDsfsapikey();
        if (empty($key)) {
            $message = "A organização $nrOrg não possui uma API Key do File Server!";
            throw new \Exception($message);
        }

        return $key;
    }

    public function upload($params) {
        $path = $this->client->uploadFile($params);

        return $path;
    }

    /**
    * Requests the FileServer the file with the respective path.
    * @param string $path A string containg the path to the file once stored at FileServer.
    * @return array Array with the file in base64 format and its MIME type.
    */
    public function download(string $path): array {
        $file = $this->client->getFile($path);

        return $file;
    }

    public function delete(string $path): bool {
        $deleted = $this->client->deleteFile($path);

        return $deleted;
    }
}